@@ -37,6 +37,7 @@
 #define ADJUSTMENT_MAX_NORMAL 65536.0 /* PA_VOLUME_NORM */
 #define ADJUSTMENT_MAX_AMPLIFIED 98304.0 /* 1.5 * ADJUSTMENT_MAX_NORMAL */
 #define ADJUSTMENT_MAX (bar->priv->is_amplified ? ADJUSTMENT_MAX_AMPLIFIED : ADJUSTMENT_MAX_NORMAL)
+#define SCROLLSTEP (ADJUSTMENT_MAX / 100.0 * 5.0)
 
 #define GVC_CHANNEL_BAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GVC_TYPE_CHANNEL_BAR, GvcChannelBarPrivate))
 
@@ -157,7 +158,6 @@ _scale_box_new (GvcChannelBar *bar)
         }
 
         gtk_range_set_update_policy (GTK_RANGE (priv->scale), GTK_UPDATE_CONTINUOUS);
-        gtk_scale_set_digits (GTK_SCALE (priv->scale), 0);
         ca_gtk_widget_disable_sounds (bar->priv->scale, FALSE);
         gtk_widget_add_events (bar->priv->scale, GDK_SCROLL_MASK);
 
@@ -198,12 +198,14 @@ update_image (GvcChannelBar *bar)
 static void
 update_label (GvcChannelBar *bar)
 {
-        gtk_label_set_text (GTK_LABEL (bar->priv->label),
-                            bar->priv->name);
-
         if (bar->priv->name != NULL) {
+                gtk_label_set_text_with_mnemonic (GTK_LABEL (bar->priv->label),
+                                                  bar->priv->name);
+                gtk_label_set_mnemonic_widget (GTK_LABEL (bar->priv->label),
+                                               bar->priv->scale);
                 gtk_widget_show (bar->priv->label);
         } else {
+                gtk_label_set_text (GTK_LABEL (bar->priv->label), NULL);
                 gtk_widget_hide (bar->priv->label);
         }
 }
@@ -411,11 +413,8 @@ on_scale_button_release_event (GtkWidget      *widget,
         value = gtk_adjustment_get_value (adj);
 
         /* this means the adjustment moved away from zero and
-          therefore we should unmute and set the volume. */
-        if (value > 0)
-                gvc_channel_bar_set_is_muted (bar, FALSE);
-        else
-                gvc_channel_bar_set_is_muted (bar, TRUE);
+         * therefore we should unmute and set the volume. */
+        gvc_channel_bar_set_is_muted (bar, (value == 0.0));
 
         /* Play a sound! */
         ca_gtk_play_for_widget (GTK_WIDGET (bar), 0,
@@ -450,15 +449,15 @@ gvc_channel_bar_scroll (GvcChannelBar *bar, GdkScrollDirection direction)
         value = gtk_adjustment_get_value (adj);
 
         if (direction == GDK_SCROLL_UP) {
-                if (value + ADJUSTMENT_MAX/100.0 > ADJUSTMENT_MAX)
+                if (value + SCROLLSTEP > ADJUSTMENT_MAX)
                         value = ADJUSTMENT_MAX;
                 else
-                        value = value + ADJUSTMENT_MAX/100.0;
+                        value = value + SCROLLSTEP;
         } else if (direction == GDK_SCROLL_DOWN) {
-                if (value - ADJUSTMENT_MAX/100.0 < 0)
+                if (value - SCROLLSTEP < 0)
                         value = 0.0;
                 else
-                        value = value - ADJUSTMENT_MAX/100.0;
+                        value = value - SCROLLSTEP;
         }
 
         gvc_channel_bar_set_is_muted (bar, (value == 0.0));
@@ -489,10 +488,12 @@ on_zero_adjustment_value_changed (GtkAdjustment *adjustment,
         value = gtk_adjustment_get_value (bar->priv->zero_adjustment);
         gtk_adjustment_set_value (bar->priv->adjustment, value);
 
-        /* this means the adjustment moved away from zero and
-          therefore we should unmute and set the volume. */
 
-        gvc_channel_bar_set_is_muted (bar, FALSE);
+        if (bar->priv->show_mute == FALSE) {
+                /* this means the adjustment moved away from zero and
+                 * therefore we should unmute and set the volume. */
+                gvc_channel_bar_set_is_muted (bar, value > 0.0);
+        }
 }
 
 static void
@@ -519,9 +520,11 @@ update_mute_button (GvcChannelBar *bar)
                         gtk_range_set_adjustment (GTK_RANGE (bar->priv->scale),
                                                   bar->priv->zero_adjustment);
                 } else {
-                        /* no longer muted so restore the original adjustment */
+                        /* no longer muted so restore the original adjustment
+                         * and tell the front-end that the value changed */
                         gtk_range_set_adjustment (GTK_RANGE (bar->priv->scale),
                                                   bar->priv->adjustment);
+                        gtk_adjustment_value_changed (bar->priv->adjustment);
                 }
         }
 }
@@ -533,9 +536,11 @@ gvc_channel_bar_set_is_muted (GvcChannelBar *bar,
         g_return_if_fail (GVC_IS_CHANNEL_BAR (bar));
 
         if (is_muted != bar->priv->is_muted) {
+                /* Update our internal state before telling the
+                 * front-end about our changes */
                 bar->priv->is_muted = is_muted;
-                g_object_notify (G_OBJECT (bar), "is-muted");
                 update_mute_button (bar);
+                g_object_notify (G_OBJECT (bar), "is-muted");
         }
 }
 
@@ -587,6 +592,12 @@ gvc_channel_bar_set_is_amplified (GvcChannelBar *bar, gboolean amplified)
                         str = g_strdup_printf ("<small>%s</small>", C_("volume", "Unamplified"));
                         gtk_scale_add_mark (GTK_SCALE (bar->priv->scale), bar->priv->base_volume,
                                             GTK_POS_BOTTOM, str);
+                        /* Only show 100% if it's higher than the base volume */
+                        if (bar->priv->base_volume < ADJUSTMENT_MAX_NORMAL) {
+                                str = g_strdup_printf ("<small>%s</small>", C_("volume", "100%"));
+                                gtk_scale_add_mark (GTK_SCALE (bar->priv->scale), ADJUSTMENT_MAX_NORMAL,
+                                                    GTK_POS_BOTTOM, str);
+                        }
                 }
 
                 g_free (str);
@@ -611,7 +622,7 @@ gvc_channel_bar_set_base_volume (GvcChannelBar *bar,
         if (base_volume == 0) {
                 bar->priv->base_volume = ADJUSTMENT_MAX_NORMAL;
                 return;
-	}
+        }
 
         /* Note that you need to call _is_amplified() afterwards to update the marks */
         bar->priv->base_volume = base_volume;
