@@ -165,7 +165,6 @@ static void
 bar_calc_layout (GvcLevelBar *bar)
 {
         GdkColor color;
-        int      rms_level;
         int      peak_level;
         int      max_peak_level;
 
@@ -186,7 +185,6 @@ bar_calc_layout (GvcLevelBar *bar)
         bar->priv->layout.fl_b = (float)color.blue / 65535.0;
 
         if (bar->priv->orientation == GTK_ORIENTATION_VERTICAL) {
-                rms_level = bar->priv->rms_fraction * bar->priv->layout.area.height;
                 peak_level = bar->priv->peak_fraction * bar->priv->layout.area.height;
                 max_peak_level = bar->priv->max_peak * bar->priv->layout.area.height;
 
@@ -197,7 +195,6 @@ bar_calc_layout (GvcLevelBar *bar)
                 bar->priv->layout.box_width = bar->priv->layout.area.width;
                 bar->priv->layout.box_radius = bar->priv->layout.box_width / 2;
         } else {
-                rms_level = bar->priv->rms_fraction * bar->priv->layout.area.width;
                 peak_level = bar->priv->peak_fraction * bar->priv->layout.area.width;
                 max_peak_level = bar->priv->max_peak * bar->priv->layout.area.width;
 
@@ -426,14 +423,7 @@ gvc_level_bar_constructor (GType                  type,
                            guint                  n_construct_properties,
                            GObjectConstructParam *construct_params)
 {
-        GObject     *object;
-        GvcLevelBar *self;
-
-        object = G_OBJECT_CLASS (gvc_level_bar_parent_class)->constructor (type, n_construct_properties, construct_params);
-
-        self = GVC_LEVEL_BAR (object);
-
-        return object;
+        return G_OBJECT_CLASS (gvc_level_bar_parent_class)->constructor (type, n_construct_properties, construct_params);
 }
 
 static void
@@ -576,10 +566,10 @@ gvc_level_bar_expose (GtkWidget      *widget,
                 for (i = 0; i < NUM_BOXES; i++) {
                         by = i * bar->priv->layout.delta;
                         curved_rectangle (cr,
-                                          bar->priv->layout.area.x,
-                                          by,
-                                          bar->priv->layout.box_width,
-                                          bar->priv->layout.box_height,
+                                          bar->priv->layout.area.x + 0.5,
+                                          by + 0.5,
+                                          bar->priv->layout.box_width - 1,
+                                          bar->priv->layout.box_height - 1,
                                           bar->priv->layout.box_radius);
                         if ((bar->priv->layout.max_peak_num - 1) == i) {
                                 /* fill peak foreground */
@@ -611,10 +601,10 @@ gvc_level_bar_expose (GtkWidget      *widget,
                 for (i = 0; i < NUM_BOXES; i++) {
                         bx = i * bar->priv->layout.delta;
                         curved_rectangle (cr,
-                                          bx,
-                                          bar->priv->layout.area.y,
-                                          bar->priv->layout.box_width,
-                                          bar->priv->layout.box_height,
+                                          bx + 0.5,
+                                          bar->priv->layout.area.y + 0.5,
+                                          bar->priv->layout.box_width - 1,
+                                          bar->priv->layout.box_height - 1,
                                           bar->priv->layout.box_radius);
 
                         if ((bar->priv->layout.max_peak_num - 1) == i) {
@@ -730,14 +720,18 @@ gvc_level_bar_init (GvcLevelBar *bar)
 static void
 gvc_level_bar_finalize (GObject *object)
 {
-        GvcLevelBar *level_bar;
+        GvcLevelBar *bar;
 
         g_return_if_fail (object != NULL);
         g_return_if_fail (GVC_IS_LEVEL_BAR (object));
 
-        level_bar = GVC_LEVEL_BAR (object);
+        bar = GVC_LEVEL_BAR (object);
 
-        g_return_if_fail (level_bar->priv != NULL);
+        if (bar->priv->max_peak_id > 0) {
+                g_source_remove (bar->priv->max_peak_id);
+        }
+
+        g_return_if_fail (bar->priv != NULL);
 
         G_OBJECT_CLASS (gvc_level_bar_parent_class)->finalize (object);
 }
