@@ -70,7 +70,11 @@ gnome_volume_control_volume_class_init (GnomeVolumeControlVolumeClass *klass)
 static void
 gnome_volume_control_volume_init (GnomeVolumeControlVolume *vol)
 {
+  #if GTK_CHECK_VERSION(2,18,0)
+  gtk_widget_set_has_window (GTK_WIDGET (vol), TRUE);
+  #else
   gtk_fixed_set_has_window (GTK_FIXED (vol), TRUE);
+  #endif
 
   vol->mixer = NULL;
   vol->track = NULL;
@@ -164,7 +168,6 @@ gnome_volume_control_volume_new (GstMixer *mixer,
   GnomeVolumeControlVolume *vol;
   gint *volumes, n;
   gchar *msg, *chan;
-  GList *items;
   gboolean need_timeout = TRUE;
 
   need_timeout = ((gst_mixer_get_mixer_flags (GST_MIXER (mixer)) &
@@ -278,7 +281,6 @@ gnome_volume_control_volume_size_req (GtkWidget *widget,
 {
   GnomeVolumeControlVolume *vol = GNOME_VOLUME_CONTROL_VOLUME (widget);
   GtkRequisition but_req, scale_req;
-  gint w,h;
 
   /* request size of kids */
   GTK_WIDGET_GET_CLASS (vol->button)->size_request (vol->button, &but_req);
@@ -300,7 +302,7 @@ gnome_volume_control_volume_size_alloc (GtkWidget *widget,
   GnomeVolumeControlVolume *vol = GNOME_VOLUME_CONTROL_VOLUME (widget);
   GtkRequisition but_req, scale_req;
   GtkAllocation but_all, scale_all;
-  gint x_offset, but_deco_y_offset, but_x_offset, but_deco_width, n = 0;
+  gint x_offset, but_deco_width, n = 0;
   GList *scales;
 
   /* loop? */
